# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['braid_db',
 'braid_db.funcx',
 'braid_db.models',
 'braid_db.models.api',
 'braid_db.service.viz',
 'braid_db.tools',
 'workflows',
 'workflows.BraggNN',
 'workflows.CTSegNet',
 'workflows.SLAC',
 'workflows.SSX',
 'workflows.mascot']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy==1.4.35',
 'funcx-endpoint>=1,<2',
 'funcx>=1,<2',
 'simpleeval>=0.9,<0.10',
 'sqlmodel>=0,<1']

entry_points = \
{'console_scripts': ['db-create = braid_db.tools.db_create:main',
                     'db-print = braid_db.tools.db_print:main',
                     'db-to-dot = braid_db.service.viz.dot_graphing:main',
                     'db-to-mermaid = '
                     'braid_db.service.viz.mermaid_graphing:main',
                     'funcx-add-record = '
                     'braid_db.funcx.funcx_main:funcx_add_record',
                     'invalidate-record = '
                     'braid_db.tools.record_invalidate:main',
                     'register-funcx = '
                     'braid_db.funcx.funcx_main:register_functions',
                     'workflow-BraggNN = workflows.BraggNN.workflow:main',
                     'workflow-CTSegNet = workflows.CTSegNet.workflow:main',
                     'workflow-SLAC = workflows.SLAC.workflow:main',
                     'workflow-SSX = workflows.SSX.workflow:main']}

setup_kwargs = {
    'name': 'braid-db',
    'version': '0.1.0a1',
    'description': 'The Braid Project Provenance DB',
    'long_description': '\n= DB\n\nBraid provenance system\n\n== System goals\n\n* Embrace automation in data analysis, retention, decision-making\n* Enable users to trace back to how decisions were made\n* Necessitates recording what went into model training, including external data, simulations, and structures of other learning and analysis activity\n* Envision a versioned database for ML model states with HPC interfaces\n* Develop recursive and versioned provenance structures:\n** Models may be constructed via other models (estimates, surrogates)\n** Models are constantly updated (track past decisions and allow updates)\n* Integrate with other Braid components\n\n== Architecture\n\n:imagesdir: https://github.com/ANL-Braid/DB/raw/main/img\n\n=== Conceptual architecture\n\nimage::conceptual-architecture.png[]\n\n=== Software block diagram\n\nimage::block-diagram.png[]\n\n== Use cases\n\n=== SLAC workflow\n\n==== Goals\n\n. Perform data reduction at the edge\n. Train models on specific characteristics of experiment-specific data\n\n==== Workflow\n\n. Scientist configures experiment parameters\n. Workflow launches simulations with experiment parameters\n. Complete simulations by time experiment data collection is complete\n. Train model on simulation and experiment data\n. Run model on an FPGA to perform data reduction in production\n\n==== Provenance records\n\n===== Notes\n\n* Everything has typical metadata like timestamps\n* Records can be updated\n** Not immutable history like most provenance data\n** Old versions can be recovered and used\n\n===== Records\n\n. Experimental configurations (independent?)\n. Experiment outputs\n. Other simulation inputs?\n.. Software version, configuration?\n. Simulation outputs\n. Training data ingest\n. Inference outputs (statements)\n.. Could be in the form of tests\n.. Like a super-Jenkins\n\n=== BraggNN workflow\n\n==== Goals\n\n. Improve peak finding\n. Train model to represent Bragg peaks\n\n==== Workflow\n\n. Scientist configures experiment parameters\n. APS collects raw scattering data\n. Run peak finding on raw data, label peaks\n. Train model on peaks to represent raw data\n. Reproduce and save peak locations\n\n==== Provenance records\n\n===== Notes\n\n* Everything has typical metadata like timestamps\n\n===== Records\n\n. Experimental configurations (independent?)\n. Experiment outputs\n. Derived peak locations\n. Models trained, checkpoints, etc.\n. Inferred peak locations from trained model\n\n=== SSX\n\n==== Goals\n\n. Track the provenance of SSX crystal structures\n\n==== Workflow\n\n. Scientists create a beamline.json and process.phil file\n. Analysis is performed on the input data using these configs to create int files\n. The int files are used with a prime.phil file to create a structure\n\n==== Provenance records\n\n===== Notes\n\n. Structures can come from multiple experiments. This is defined by an intlist in the prime.phil file.\n\n===== Records\n\n. Experimental config files (phil, beamline.json)\n. Analysis results (int files)\n. Derived structure\n\n=== CTSegNet\n\n==== Goals\n\n. Track the history of various U-Net-like models used for trial-and-error image segmentation\n\n==== Workflow\n\nhttps://raw.githubusercontent.com/aniketkt/CTSegNet/master/notebooks/CTSegNet_continuouslearning.png[Diagram]\n\n. A tomo scan is obtained\n. Perform image processing, contrast adjustment, etc.\n. Apply (labeling) "masks"\n. Run ensemble models in inference mode\n. Get new segmentations\n. Aggregate segmentation results\n. Re-train models and loop...\n\n==== Provenance notes\n\n. Models are trained on inferences of previous models\n. Provenance queries:\n.. "What data was used to train this model?"\n. Refer to TomoBank IDs for data identification scheme\n. Need rich metadata for search\n\n=== Samarakoon/Osborn\n\n==== Goals\n\n. Fit simulated crystal structure to scattering data\n\n==== Workflow\n\n. Obtain neutron scattering data\n. Apply auto-encoder to identify important features\n. Apply dimensionality reduction\n. Fit to data\n\n== Getting started\n\nNote: Additional docs forthcoming to help get setup in a (mini)conda based environment.\n\n* Start by installing poetry via pip or\nhttps://python-poetry.org/docs/#osx--linux--bashonwindows-install-instructions\n\n* Then, run `poetry install` to setup a local virtual environment from which to run other applications\n* Run, `poetry run pre-commit install` to setup pre-commit hooks for code formatting, lining, etc.\n* Tests can be run using scripts in the `test` directory.\n* Unit tests can be run with the command `poetry run pytest pytests/` which will run the pytest test driver from the current virtual environment on all then tests defined in the `pytests` directory.\n\n== Working with FuncX\n\n=== Setup\n\n. `pip install funcx funcx-endpoint`\n\n\n=== Usage\n\nUsing FuncX requires that the `funcx-endpoint` be installed in a working environment whether it is a conda, pip or otherwise installed process.\n\nSetting up to run with funcx is a multi-step process:\n\n1. Create a new funcX endpoint configured so that it can use the BraidDB library. This can be done with the shell script: `scripts/configure_funcx_endpoint.sh`. Provide one command line argument: the name of the funcx endpoint to be created/configured. If no name is provided, the endpoint will be named `braid_db`. This endpoint will be configured such that it has access to the implementation stored in the `.venv` directory of the braid db project.\n\n2. Start the new endpoint with the command `funcx-endpoint start <endpoint_name>` where `endpoint_name` is as configured in the previous step. Take note of the UUID generated for the new endpoint.\n\n3. If you want the funcx hosted braid db to store files in a different location, edit `src/braid_db/funcx/funcx_main.py` and change the value of `DB_FILE` in the function `funcx_add_record`. If not edited the funcx-based operations will store their entries in the file `~/funcx-braid.db`.\n\n4. Register the function(s) to be exposed to funcx. This can be done with the command `.venv/bin/register-funcx` (or `poetry run register-funcx`). Note that this requires that the command `poetry install` has previously been run so that the script is installed in the virtual environment (in the `.venv/bin` directory). This will register a number of functions printing the name of the function and the funcx uuid for the registered function.\n\nThe functions are:\n  * `add_record_for_action_step`: This simply adds a record to the DB which is commonly used to represent a step of a Globus Flow having been run.\n  * `add_transfer_request`: This is a helper for adding a record to the DB representing a Globus Flow step using the Globus Transfer Service. It will also create records for the source and destination of the Transfer with dependency from the source, to the workflow step to the destination.\n  * `create_invalidation_action`: Creates an invalidation action in the DB. It takes as input the operations to perform when an invalidation should occur.\n  * `add_invalidation_action_to_record`: Creates an association between a record in the DB and an invalidation action (most likely created with the `create_invalidation_action` function above.\n\nThe various functions return output in a format which places state into a running Globus Flow which can be consumed by subsequent steps which invoke other functions. For example, the `add_record_for_action_state` function returns the DB record id in its output. Subsequent calls to this function within a Flow can reference this output as a "predecessor" and the function will interpret the input such that it can determine the previous step\'s record id and create the dependency between the newly created record and the previous record.\n\n\n5. *Deprecated at this time* To test invoking the add record function via funcx, run the command: `.venv/bin/funcx-add-record --endpoint-id <endpoint_id> --function-id <function_id>` using the value for endpoint id and function id output in the previous steps. This should output the record id. One can use a tool like `sqlite3` to verify that records are stored in the database file.  *Deprecated at this time*\n\n== Developer notes\n\n* There is a high-level SQL API wrapper in db_tools called BraidSQL. +\n  This API is generic SQL, it does not know about Braid concepts\n* The high-level Braid Database API is called BraidDB\n* BraidDB is used by the Braid concepts: BraidFact, BraidRecord,\n  BraidModel, ...\n* The Braid concepts are used by the workflows\n* We constantly check the DB connection because this is useful when\n  running workflows\n\n== Tools\n\nbin/braid-db-create::\nCreates a DB based on the structure in braid-db.sql\n\nbin/braid-db-print::\nPrint the DB to text\n\n== Tests\n\nTests are in the test/ directory.\n\nTests are run nightly at:\n\n* https://jenkins-ci.cels.anl.gov/job/Braid-DB-Core\n* https://jenkins-ci.cels.anl.gov/job/Braid-DB-Workflows\n\nThey are also run via Github Actions for each push or pull request against the origin repo.\n',
    'author': 'Justin Wozniak',
    'author_email': 'woz@anl.gov',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ANL-Braid/DB',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
